@@ -39,7 +39,7 @@
 static FILE *info_file;
 
 static int map_one_extent(struct btrfs_fs_info *fs_info,
-			  u64 *logical_ret, u64 *len_ret, int search_foward)
+			  u64 *logical_ret, u64 *len_ret, int search_forward)
 {
 	struct btrfs_path *path;
 	struct btrfs_key key;
@@ -67,15 +67,16 @@ static int map_one_extent(struct btrfs_fs_info *fs_info,
 
 again:
 	btrfs_item_key_to_cpu(path->nodes[0], &key, path->slots[0]);
-	if ((search_foward && key.objectid < logical) ||
-	    (!search_foward && key.objectid > logical) ||
+	if ((search_forward && key.objectid < logical) ||
+	    (!search_forward && key.objectid > logical) ||
 	    (key.type != BTRFS_EXTENT_ITEM_KEY &&
 	     key.type != BTRFS_METADATA_ITEM_KEY)) {
-		if (!search_foward)
+		if (!search_forward)
 			ret = btrfs_previous_extent_item(fs_info->extent_root,
 							 path, 0);
 		else
-			ret = btrfs_next_item(fs_info->extent_root, path);
+			ret = btrfs_next_extent_item(fs_info->extent_root,
+						     path, 0);
 		if (ret)
 			goto out;
 		goto again;
@@ -190,7 +191,7 @@ static int write_extent_content(struct btrfs_fs_info *fs_info, int out_fd,
 	return ret;
 }
 
-static void print_usage(void) __attribute__((noreturn));
+__attribute__((noreturn))
 static void print_usage(void)
 {
 	printf("usage: btrfs-map-logical [options] device\n");
